module Peatio
  module Decredcoin
    class Wallet < Peatio::Wallet::Abstract

      DEFAULT_FEATURES = { skip_deposit_collection: false }.freeze
      SUPPORTED_FEATURES = {case_sensitive: true, cash_addr_format: false}.freeze

      def initialize(custom_features = {})
        @features = DEFAULT_FEATURES.merge(custom_features).slice(*SUPPORTED_FEATURES)
        @settings = {}
      end

      def configure(settings = {})
        # Clean client state during configure.
        @client = nil

        @settings.merge!(settings.slice(*SUPPORTED_SETTINGS))

        @wallet = @settings.fetch(:wallet) do
          raise Peatio::Wallet::MissingSettingError, :wallet
        end.slice(:uri, :address, :passphrase)

        @currency = @settings.fetch(:currency) do
          raise Peatio::Wallet::MissingSettingError, :currency
        end.slice(:id, :base_factor, :options)
      end

      def create_address!(_options = {})
        { address: client.json_rpc(:getnewaddress) }
      rescue Decredcoin::Client::Error => e
        raise Peatio::Wallet::ClientError, e
      end

      # create a transaction
      def create_transaction!(transaction, options = {})
        unlock_wallet!(options[:timeout])
        txid = client.json_rpc(:sendtoaddress,
                               [
                                   transaction.to_address,
                                   transaction.amount.to_f
                               ])
        transaction.hash = txid
        transaction
      rescue Decredcoin::Client::Error => e
        raise Peatio::Wallet::ClientError, e
      end

      def load_balance!(account_name = 'default')
        response = client.json_rpc(:getbalance)
        account = response['balances'].find { |k| k['accountname'].eql? account_name }

        raise Decredcoin::Client::Error.new 'account does not exists' unless account

        account['total'].to_d
      rescue Decredcoin::Client::Error => e
        raise Peatio::Wallet::ClientError, e
      end

      private

      # unlock wallet for 15 seconds
      def unlock_wallet!(timeout = 15)
        client.json_rpc(:walletpassphrase,
                        [
                            @wallet.fetch(:passphrase),
                            timeout
                        ])
      rescue Decredcoin::Client::Error => e
        raise Peatio::Wallet::ClientError, e
      end

      def client
        uri = @wallet.fetch(:uri) { raise Peatio::Wallet::MissingSettingError, :uri }
        @client ||= Client.new(uri)
      end
    end
  end
end
